@@ -22,23 +22,28 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "fcint.h"
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
-#include "fcint.h"
+#ifdef _WIN32
+#include <windows.h>
+#endif
 
 FcChar8 *
 FcStrCopy (const FcChar8 *s)
 {
-    FcChar8	*r;
+    int     len;
+    FcChar8 *r;
 
     if (!s)
 	return 0;
-    r = (FcChar8 *) malloc (strlen ((char *) s) + 1);
+    len = strlen ((char *) s) + 1;
+    r = (FcChar8 *) malloc (len);
     if (!r)
 	return 0;
-    FcMemAlloc (FC_MEM_STRING, strlen ((char *) s) + 1);
-    strcpy ((char *) r, (char *) s);
+    FcMemAlloc (FC_MEM_STRING, len);
+    memcpy (r, s, len);
     return r;
 }
 
@@ -706,7 +711,7 @@ FcStrBufChar (FcStrBuf *buf, FcChar8 c)
 	}
 	else
 	{
-	    size = buf->size + 1024;
+	    size = buf->size + 64;
 	    new = malloc (size);
 	    if (new)
 	    {
@@ -762,26 +767,21 @@ FcStrCopyFilename (const FcChar8 *s)
     if (*s == '~')
     {
 	FcChar8	*home = FcConfigHome ();
+	FcChar8	*full;
 	int	size;
 	if (!home)
 	    return 0;
 	size = strlen ((char *) home) + strlen ((char *) s);
-	new = (FcChar8 *) malloc (size);
-	if (!new)
+	full = (FcChar8 *) malloc (size);
+	if (!full)
 	    return 0;
-	FcMemAlloc (FC_MEM_STRING, size);
-	strcpy ((char *) new, (char *) home);
-	strcat ((char *) new, (char *) s + 1);
+	strcpy ((char *) full, (char *) home);
+	strcat ((char *) full, (char *) s + 1);
+	new = FcStrCanonFilename (full);
+	free (full);
     }
     else
-    {
-	int	size = strlen ((char *) s) + 1;
-	new = (FcChar8 *) malloc (size);
-	if (!new)
-	    return 0;
-	FcMemAlloc (FC_MEM_STRING, size);
-	strcpy ((char *) new, (const char *) s);
-    }
+	new = FcStrCanonFilename (s);
     return new;
 }
 
@@ -833,6 +833,121 @@ FcStrBasename (const FcChar8 *file)
     return FcStrCopy (slash + 1);
 }
 
+static FcChar8 *
+FcStrCanonAbsoluteFilename (const FcChar8 *s)
+{
+    FcChar8 *file;
+    FcChar8 *f;
+    const FcChar8 *slash;
+    int size;
+
+    size = strlen ((char *) s) + 1;
+    file = malloc (size);
+    if (!file)
+	return NULL;
+    FcMemAlloc (FC_MEM_STRING, size);
+    slash = NULL;
+    f = file;
+    for (;;) {
+	if (*s == '/' || *s == '\0')
+	{
+	    if (slash)
+	    {
+		switch (s - slash) {
+		case 2:
+		    if (!strncmp ((char *) slash, "/.", 2))
+		    {
+			f -= 2;	/* trim /. from file */
+		    }
+		    break;
+		case 3:
+		    if (!strncmp ((char *) slash, "/..", 3))
+		    {
+			f -= 3;	/* trim /.. from file */
+			while (f > file) {
+			    if (*--f == '/')
+				break;
+			}
+		    }
+		    break;
+		}
+	    }
+	    slash = s;
+	}
+	if (!(*f++ = *s++))
+	    break;
+    }
+    return file;
+}
+ 
+#ifdef _WIN32
+/*
+ * Convert '\\' to '/' , remove double '/' 
+ */
+static void
+FcConvertDosPath (char *str)
+{
+  size_t len = strlen (str);
+  char *p = str;
+  char *dest = str;
+  char *end = str + len;
+  char last = 0;
+  
+  while (p < end)
+    {
+      if (*p == '\\')
+	*p = '/';
+
+      if (*p != '/'
+	  || last != '/')
+	{
+	  *dest++ = *p;
+	}
+
+      last = *p;
+      p++;
+    }
+
+  *dest = 0;
+}
+#endif
+
+FcChar8 *
+FcStrCanonFilename (const FcChar8 *s)
+{
+#ifdef _WIN32
+    FcChar8 full[FC_MAX_FILE_LEN + 2];
+    FcChar8 basename[FC_MAX_FILE_LEN + 2];
+    int size = GetFullPathName (s, sizeof (full) -1,
+				full,
+				basename);
+
+    if (size == 0)
+	perror ("GetFullPathName");
+
+    FcConvertDosPath (full);
+    return FcStrCanonAbsoluteFilename (full);
+#else
+    if (s[0] == '/')
+	return FcStrCanonAbsoluteFilename (s);
+    else
+    {
+	FcChar8	*full;
+	FcChar8 *file;
+
+	FcChar8	cwd[FC_MAX_FILE_LEN + 2];
+	if (getcwd ((char *) cwd, FC_MAX_FILE_LEN) == NULL)
+	    return NULL;
+	strcat ((char *) cwd, "/");
+	full = FcStrPlus (cwd, s);
+	file = FcStrCanonAbsoluteFilename (full);
+	FcStrFree (full);
+	return file;
+    }
+#endif
+}
+
+
 FcStrSet *
 FcStrSetCreate (void)
 {
@@ -993,3 +1108,7 @@ FcStrListDone (FcStrList *list)
     FcMemFree (FC_MEM_STRLIST, sizeof (FcStrList));
     free (list);
 }
+
+#define __fcstr__
+#include "fcaliastail.h"
+#undef __fcstr__
